@@ -9,24 +9,16 @@
 
 #include <inttypes.h>
 #include <math.h>
-#include <setjmp.h>
-#include <stdarg.h>
-#include <stddef.h>
-#include <stdint.h>
-#include <string.h>
 
 #include "google/protobuf/descriptor.upb.h"
 #include "upb/base/descriptor_constants.h"
-#include "upb/base/string_view.h"
-#include "upb/mem/arena.h"
-#include "upb/message/array.h"
 #include "upb/port/vsnprintf_compat.h"
 #include "upb/reflection/def.h"
 #include "upb/reflection/enum_reserved_range.h"
 #include "upb/reflection/extension_range.h"
-#include "upb/reflection/internal/def_pool.h"
 #include "upb/reflection/internal/field_def.h"
 #include "upb/reflection/internal/file_def.h"
+#include "upb/reflection/message.h"
 #include "upb/reflection/message_reserved_range.h"
 
 // Must be last.
@@ -42,16 +34,16 @@ typedef struct {
 
 // We want to copy the options verbatim into the destination options proto.
 // We use serialize+parse as our deep copy.
-#define SET_OPTIONS(proto, desc_type, options_type, src, extreg)           \
-  {                                                                        \
-    size_t size;                                                           \
-    /* MEM: could use a temporary arena here instead. */                   \
-    char* pb = google_protobuf_##options_type##_serialize(src, ctx->arena, &size);  \
-    CHK_OOM(pb);                                                           \
-    google_protobuf_##options_type* dst =                                           \
-        google_protobuf_##options_type##_parse_ex(pb, size, extreg, 0, ctx->arena); \
-    CHK_OOM(dst);                                                          \
-    google_protobuf_##desc_type##_set_options(proto, dst);                          \
+#define SET_OPTIONS(proto, desc_type, options_type, src)                  \
+  {                                                                       \
+    size_t size;                                                          \
+    /* MEM: could use a temporary arena here instead. */                  \
+    char* pb = google_protobuf_##options_type##_serialize(src, ctx->arena, &size); \
+    CHK_OOM(pb);                                                          \
+    google_protobuf_##options_type* dst =                                          \
+        google_protobuf_##options_type##_parse(pb, size, ctx->arena);              \
+    CHK_OOM(dst);                                                         \
+    google_protobuf_##desc_type##_set_options(proto, dst);                         \
   }
 
 static upb_StringView strviewdup2(upb_ToProto_Context* ctx,
@@ -231,14 +223,14 @@ static google_protobuf_FieldDescriptorProto* fielddef_toproto(upb_ToProto_Contex
   google_protobuf_FieldDescriptorProto_set_number(proto, upb_FieldDef_Number(f));
 
   if (upb_FieldDef_IsRequired(f) &&
-      upb_FileDef_Edition(upb_FieldDef_File(f)) >= google_protobuf_EDITION_2023) {
+      upb_FileDef_Edition(upb_FieldDef_File(f)) >= UPB_DESC(EDITION_2023)) {
     google_protobuf_FieldDescriptorProto_set_label(
-        proto, google_protobuf_FieldDescriptorProto_LABEL_OPTIONAL);
+        proto, UPB_DESC(FieldDescriptorProto_LABEL_OPTIONAL));
   } else {
     google_protobuf_FieldDescriptorProto_set_label(proto, upb_FieldDef_Label(f));
   }
   if (upb_FieldDef_Type(f) == kUpb_FieldType_Group &&
-      upb_FileDef_Edition(upb_FieldDef_File(f)) >= google_protobuf_EDITION_2023) {
+      upb_FileDef_Edition(upb_FieldDef_File(f)) >= UPB_DESC(EDITION_2023)) {
     google_protobuf_FieldDescriptorProto_set_type(proto, kUpb_FieldType_Message);
   } else {
     google_protobuf_FieldDescriptorProto_set_type(proto, upb_FieldDef_Type(f));
@@ -280,9 +272,7 @@ static google_protobuf_FieldDescriptorProto* fielddef_toproto(upb_ToProto_Contex
 
   if (upb_FieldDef_HasOptions(f)) {
     SET_OPTIONS(proto, FieldDescriptorProto, FieldOptions,
-                upb_FieldDef_Options(f),
-                _upb_DefPool_GeneratedExtensionRegistry(
-                    upb_FileDef_Pool(upb_FieldDef_File(f))));
+                upb_FieldDef_Options(f));
   }
 
   return proto;
@@ -299,9 +289,7 @@ static google_protobuf_OneofDescriptorProto* oneofdef_toproto(upb_ToProto_Contex
 
   if (upb_OneofDef_HasOptions(o)) {
     SET_OPTIONS(proto, OneofDescriptorProto, OneofOptions,
-                upb_OneofDef_Options(o),
-                _upb_DefPool_GeneratedExtensionRegistry(upb_FileDef_Pool(
-                    upb_MessageDef_File(upb_OneofDef_ContainingType(o)))));
+                upb_OneofDef_Options(o));
   }
 
   return proto;
@@ -319,9 +307,7 @@ static google_protobuf_EnumValueDescriptorProto* enumvaldef_toproto(
 
   if (upb_EnumValueDef_HasOptions(e)) {
     SET_OPTIONS(proto, EnumValueDescriptorProto, EnumValueOptions,
-                upb_EnumValueDef_Options(e),
-                _upb_DefPool_GeneratedExtensionRegistry(upb_FileDef_Pool(
-                    upb_EnumDef_File(upb_EnumValueDef_Enum(e)))));
+                upb_EnumValueDef_Options(e));
   }
 
   return proto;
@@ -359,13 +345,12 @@ static google_protobuf_EnumDescriptorProto* enumdef_toproto(upb_ToProto_Context*
   }
 
   if (upb_EnumDef_HasOptions(e)) {
-    SET_OPTIONS(proto, EnumDescriptorProto, EnumOptions, upb_EnumDef_Options(e),
-                _upb_DefPool_GeneratedExtensionRegistry(
-                    upb_FileDef_Pool(upb_EnumDef_File(e))));
+    SET_OPTIONS(proto, EnumDescriptorProto, EnumOptions,
+                upb_EnumDef_Options(e));
   }
 
-  google_protobuf_SymbolVisibility visibility = upb_EnumDef_Visibility(e);
-  if (visibility != google_protobuf_VISIBILITY_UNSET) {
+  UPB_DESC(SymbolVisibility) visibility = upb_EnumDef_Visibility(e);
+  if (visibility != UPB_DESC(VISIBILITY_UNSET)) {
     google_protobuf_EnumDescriptorProto_set_visibility(proto, visibility);
   }
 
@@ -373,8 +358,7 @@ static google_protobuf_EnumDescriptorProto* enumdef_toproto(upb_ToProto_Context*
 }
 
 static google_protobuf_DescriptorProto_ExtensionRange* extrange_toproto(
-    upb_ToProto_Context* ctx, const upb_MessageDef* m,
-    const upb_ExtensionRange* e) {
+    upb_ToProto_Context* ctx, const upb_ExtensionRange* e) {
   google_protobuf_DescriptorProto_ExtensionRange* proto =
       google_protobuf_DescriptorProto_ExtensionRange_new(ctx->arena);
   CHK_OOM(proto);
@@ -386,9 +370,7 @@ static google_protobuf_DescriptorProto_ExtensionRange* extrange_toproto(
 
   if (upb_ExtensionRange_HasOptions(e)) {
     SET_OPTIONS(proto, DescriptorProto_ExtensionRange, ExtensionRangeOptions,
-                upb_ExtensionRange_Options(e),
-                _upb_DefPool_GeneratedExtensionRegistry(
-                    upb_FileDef_Pool(upb_MessageDef_File(m))));
+                upb_ExtensionRange_Options(e));
   }
 
   return proto;
@@ -445,8 +427,7 @@ static google_protobuf_DescriptorProto* msgdef_toproto(upb_ToProto_Context* ctx,
   google_protobuf_DescriptorProto_ExtensionRange** ext_ranges =
       google_protobuf_DescriptorProto_resize_extension_range(proto, n, ctx->arena);
   for (int i = 0; i < n; i++) {
-    ext_ranges[i] =
-        extrange_toproto(ctx, m, upb_MessageDef_ExtensionRange(m, i));
+    ext_ranges[i] = extrange_toproto(ctx, upb_MessageDef_ExtensionRange(m, i));
   }
 
   n = upb_MessageDef_ReservedRangeCount(m);
@@ -465,13 +446,11 @@ static google_protobuf_DescriptorProto* msgdef_toproto(upb_ToProto_Context* ctx,
 
   if (upb_MessageDef_HasOptions(m)) {
     SET_OPTIONS(proto, DescriptorProto, MessageOptions,
-                upb_MessageDef_Options(m),
-                _upb_DefPool_GeneratedExtensionRegistry(
-                    upb_FileDef_Pool(upb_MessageDef_File(m))));
+                upb_MessageDef_Options(m));
   }
 
-  google_protobuf_SymbolVisibility visibility = upb_MessageDef_Visibility(m);
-  if (visibility != google_protobuf_VISIBILITY_UNSET) {
+  UPB_DESC(SymbolVisibility) visibility = upb_MessageDef_Visibility(m);
+  if (visibility != UPB_DESC(VISIBILITY_UNSET)) {
     google_protobuf_DescriptorProto_set_visibility(proto, visibility);
   }
 
@@ -503,10 +482,8 @@ static google_protobuf_MethodDescriptorProto* methoddef_toproto(upb_ToProto_Cont
   }
 
   if (upb_MethodDef_HasOptions(m)) {
-    SET_OPTIONS(
-        proto, MethodDescriptorProto, MethodOptions, upb_MethodDef_Options(m),
-        _upb_DefPool_GeneratedExtensionRegistry(
-            upb_FileDef_Pool(upb_ServiceDef_File(upb_MethodDef_Service(m)))));
+    SET_OPTIONS(proto, MethodDescriptorProto, MethodOptions,
+                upb_MethodDef_Options(m));
   }
 
   return proto;
@@ -530,9 +507,7 @@ static google_protobuf_ServiceDescriptorProto* servicedef_toproto(
 
   if (upb_ServiceDef_HasOptions(s)) {
     SET_OPTIONS(proto, ServiceDescriptorProto, ServiceOptions,
-                upb_ServiceDef_Options(s),
-                _upb_DefPool_GeneratedExtensionRegistry(
-                    upb_FileDef_Pool(upb_ServiceDef_File(s))));
+                upb_ServiceDef_Options(s));
   }
 
   return proto;
@@ -555,20 +530,14 @@ static google_protobuf_FileDescriptorProto* filedef_toproto(upb_ToProto_Context*
     }
   }
 
-  google_protobuf_Edition edition = upb_FileDef_Edition(f);
+  if (upb_FileDef_Syntax(f) == kUpb_Syntax_Editions) {
+    google_protobuf_FileDescriptorProto_set_edition(proto, upb_FileDef_Edition(f));
+  }
 
-  switch (edition) {
-    case google_protobuf_EDITION_PROTO2:
-      // We could set syntax to "proto2", but C++ omits it completely, which is
-      // equivalent.
-      break;
-    case google_protobuf_EDITION_PROTO3:
-      google_protobuf_FileDescriptorProto_set_syntax(proto, strviewdup(ctx, "proto3"));
-      break;
-    default:
-      google_protobuf_FileDescriptorProto_set_syntax(proto, strviewdup(ctx, "editions"));
-      google_protobuf_FileDescriptorProto_set_edition(proto, edition);
-      break;
+  if (upb_FileDef_Syntax(f) == kUpb_Syntax_Proto3) {
+    google_protobuf_FileDescriptorProto_set_syntax(proto, strviewdup(ctx, "proto3"));
+  } else if (upb_FileDef_Syntax(f) == kUpb_Syntax_Editions) {
+    google_protobuf_FileDescriptorProto_set_syntax(proto, strviewdup(ctx, "editions"));
   }
 
   size_t n;
@@ -620,8 +589,8 @@ static google_protobuf_FileDescriptorProto* filedef_toproto(upb_ToProto_Context*
   }
 
   if (upb_FileDef_HasOptions(f)) {
-    SET_OPTIONS(proto, FileDescriptorProto, FileOptions, upb_FileDef_Options(f),
-                _upb_DefPool_GeneratedExtensionRegistry(upb_FileDef_Pool(f)));
+    SET_OPTIONS(proto, FileDescriptorProto, FileOptions,
+                upb_FileDef_Options(f));
   }
 
   return proto;
